@@ -32,7 +32,8 @@ from urllib.parse import urlsplit, urljoin, urlunsplit
 
 import pkg_resources
 import requests
-from warcio import ArchiveIterator
+from warcio import ArchiveIterator, StatusAndHeaders
+from warcio.recordbuilder import RecordBuilder
 from libzim.writer import Article, Blob
 from zimscraperlib.zim.creator import Creator
 from zimscraperlib.i18n import setlocale, get_language_details, Locale
@@ -66,6 +67,23 @@ HEAD_INS = re.compile(b"(<head>)", re.I)
 DEFAULT_TAGS = ["_ftindex:yes", "_category:other", "_sw:yes"]
 
 
+FUZZY_RULES = [
+    {
+        "match": re.compile(
+            r"//.*googlevideo.com/(videoplayback\?).*(id=[^&]+).*([&]itag=[^&]+).*"
+        ),
+        "replace": r"//youtube.fuzzy.replayweb.page/\1\2\3",
+    },
+    {
+        "match": re.compile(
+            r"//(?:www\.)?youtube(?:-nocookie)?\.com/(get_video_info\?).*(video_id=[^&]+).*"
+        ),
+        "replace": r"//youtube.fuzzy.replayweb.page/\1\2",
+    },
+    {"match": re.compile(r"(\.[^?]+\?)[\d]+$"), "replace": r"\1"},
+]
+
+
 # ============================================================================
 class BaseArticle(Article):
     """BaseArticle for all ZIM Articles in warc2zim with default settings"""
@@ -214,19 +232,26 @@ class StaticArticle(BaseArticle):
 
 
 # ============================================================================
-class FaviconRedirectArticle(BaseArticle):
-    def __init__(self, favicon_url):
+class RedirectArticle(BaseArticle):
+    def __init__(self, from_url, to_url):
         super().__init__()
-        self.favicon_url = favicon_url
+        self.from_url = from_url
+        self.to_url = to_url
 
     def get_url(self):
-        return "-/favicon"
+        return self.from_url
 
     def is_redirect(self):
         return True
 
     def get_redirect_url(self):
-        return "A/" + canonicalize(self.favicon_url)
+        return self.to_url
+
+
+# ============================================================================
+class FaviconRedirectArticle(RedirectArticle):
+    def __init__(self, favicon_url):
+        super().__init__("-/favicon", "A/" + canonicalize(favicon_url))
 
 
 # ============================================================================
@@ -542,6 +567,29 @@ class WARC2Zim:
         ):
             self.revisits[url] = record
 
+        self.add_fuzzy_match_record(url)
+
+    def add_fuzzy_match_record(self, url):
+        fuzzy_url = url
+        for rule in FUZZY_RULES:
+            fuzzy_url = rule["match"].sub(rule["replace"], url)
+            if fuzzy_url != url:
+                break
+
+        if fuzzy_url == url:
+            return
+
+        http_headers = StatusAndHeaders("302 Redirect", {"Location": url})
+
+        date = datetime.datetime.utcnow().isoformat()
+        builder = RecordBuilder()
+        record = builder.create_revisit_record(
+            fuzzy_url, "3I42H3S6NNFQ2MSVX7XZKYAYSCX5QBYJ", url, date, http_headers
+        )
+
+        self.revisits[fuzzy_url] = record
+        logger.debug("Adding fuzzy redirect {0} -> {1}".format(fuzzy_url, url))
+
 
 # ============================================================================
 def get_record_mime_type(record):
@@ -631,13 +679,15 @@ If not found in the ZIM, will attempt to load directly""",
 # ============================================================================
 def canonicalize(url):
     """Return a 'canonical' version of the url under which it is stored in the ZIM
-    For now, just removing the scheme
+    For now, just removing the scheme http:// or https:// scheme
     """
-    try:
-        return url.split("//", 2)[1]
-    except IndexError:
-        # likely a relative url, return as is
-        return url
+    if url.startswith("https://"):
+        return url[8:]
+
+    if url.startswith("http://"):
+        return url[7:]
+
+    return url
 
 
 # ============================================================================
